import os
from urlparse import urlparse

from charmhelpers.core import hookenv
from charmhelpers.core.host import lsb_release
from charmhelpers.core.services.base import ServiceManager
from charmhelpers.core.services import helpers

import actions
import wp_helpers


def manage():
    config = hookenv.config()
    release = lsb_release()['DISTRIB_RELEASE']
    if release < "16.04":
        php_apache_confdir = '/etc/php5/apache2/conf.d'
    else:
        php_apache_confdir = '/etc/php/7.0/apache2/conf.d/'
    manager = ServiceManager([
        {
            'service': 'wp-db',
            'required_data': [
                helpers.MysqlRelation(),
                {
                    'outbound_http_proxy': urlparse(config['outbound_http_proxy'])
                },
                wp_helpers.get_secrets_from_leader(),
            ],
            'data_ready': [
                helpers.render_template(source='wp-info.php.j2',
                                        target=os.path.join(
                                            config['install_path'],
                                            'wp-info.php')),
                actions.wordpress_static,  # wp-overrides.php, wp-config.php
            ],
            'data_lost': [actions.wipe_secrets],
            'start': [],
            'stop': [],
        },
        {
            'service': 'wp-first-install',
            'required_data': [
                helpers.MysqlRelation(),
                wp_helpers.WebserviceRelation(),
                wp_helpers.vhost_ready(),
            ],
            # We provide data here in case dedicated wp-webservice
            # service has not been called yet
            'provided_data': [wp_helpers.WebserviceRelation()],
            'data_ready': [actions.first_install],
            'start': [],
            'stop': [],
        },
        {
            'service': 'wp-config',
            'required_data': [config],
            'data_ready': [
                actions.basenode,
                actions.install_packages,
                actions.unpack_code,
                actions.notify_website_relation,
                actions.notify_webservice_relation,
                actions.apparmor_dirs,
                helpers.render_template(source='wp-apparmor.j2',
                                        target='/etc/apparmor.d/apache2.d/'
                                               'wordpress')
            ],
            'start': [actions.reload_apparmor],
            'stop': [],
        },
        {
            'service': 'wp-akismet',
            'required_data': [
                helpers.MysqlRelation(),
                wp_helpers.wordpress_configured(),
                helpers.RequiredConfig('akismet_key'),
            ],
            'data_ready': [actions.enable_akismet],
            'start': [],
            'stop': [],
        },
        {
            'service': 'wp-website',  # ie. squid-reverseproxy
            'required_data': [wp_helpers.WebsiteRelation()],
            'provided_data': [wp_helpers.WebsiteRelation()],
            'start': [],
            'stop': [],
        },
        {
            'service': 'wp-webservice',
            'required_data': [wp_helpers.WebserviceRelation()],
            'provided_data': [wp_helpers.WebserviceRelation()],
            'start': [],
            'stop': [],
        },
        {
            'service': 'wp-plugin',
            'required_data': [
                wp_helpers.PluginRelation(),
                wp_helpers.wordpress_configured(),
            ],
            'provided_data': [
                wp_helpers.PluginRelation(),
            ],
            'data_ready': [
                actions.enable_plugins,
                actions.notify_website_relation,
                actions.notify_webservice_relation,
            ],
            'start': [],
            'stop': [],
        },
        {
            'service': 'wp-theme',
            'required_data': [wp_helpers.ThemeRelation()],
            'provided_data': [wp_helpers.ThemeRelation()],
            'start': [],
            'stop': [],
        },
        {
            'service': 'wp-monitoring',
            'required_data': [
                wp_helpers.NEMRelation(),
                wp_helpers.PluginRelation().as_dict(),
                helpers.RequiredConfig(),
            ],
            'data_ready': [
                actions.write_nrpe_checks
            ],
            'data_lost': [
                actions.wipe_nrpe_checks
            ],
            'start': [],
            'stop': [],
        },
        {
            'service': 'wp-memcache',
            'required_data': [wp_helpers.MemcachedRelation(), ],
            'data_ready': [
                actions.install_php_memcache,
                helpers.render_template(source='sessionstore.ini.j2',
                                        target=os.path.join(
                                            php_apache_confdir,
                                            '50-sessionstore.ini')),
                actions.reload_apache,
            ],
            'data_lost': [
                helpers.render_template(source='sessionstore.ini.j2',
                                        target=os.path.join(
                                            php_apache_confdir,
                                            '50-sessionstore.ini')),
                actions.reload_apache,
            ],
            'start': [],
            'stop': [],
        },
    ])
    manager.manage()
